@@ -1086,12 +1086,12 @@ svytable.survey.design<-function(formula, design, Ntotal=NULL, round=FALSE,...){
   tbl
 }
 
-svycoxph<-function(formula,design,subset=NULL,...){
+svycoxph<-function(formula,design,subset=NULL,rescale=TRUE,...){
   .svycheck(design)
   UseMethod("svycoxph",design)
 }
 
-svycoxph.survey.design<-function(formula,design,subset=NULL,...){
+svycoxph.survey.design<-function(formula,design, subset=NULL, rescale=TRUE, ...){
     subset<-substitute(subset)
     subset<-eval(subset, model.frame(design),parent.frame())
     if (!is.null(subset))
@@ -1105,6 +1105,7 @@ svycoxph.survey.design<-function(formula,design,subset=NULL,...){
     g$formula<-eval.parent(g$formula)
     g$design<-NULL
     g$var<-NULL
+    g$rescale <- NULL
     if (is.null(g$weights))
       g$weights<-quote(.survey.prob.weights)
     else
@@ -1115,7 +1116,9 @@ svycoxph.survey.design<-function(formula,design,subset=NULL,...){
     g$model <- TRUE
     
     ##need to rescale weights for stability
-    data$.survey.prob.weights<-(1/design$prob)/mean(1/design$prob)
+    ## unless the user doesn't want to
+    if (rescale)
+        data$.survey.prob.weights<-(1/design$prob)/mean(1/design$prob)
     if (!all(all.vars(formula) %in% names(data))) 
         stop("all variables must be in design= argument")
     g<-with(list(data=data), eval(g))
@@ -1250,7 +1253,7 @@ svyglm<-function(formula, design,subset=NULL,family=stats::gaussian(),start=NULL
   UseMethod("svyglm",design)
 }
 
-svyglm.survey.design<-function(formula,design,subset=NULL, family=stats::gaussian(),start=NULL,...){
+svyglm.survey.design<-function(formula,design,subset=NULL, family=stats::gaussian(),start=NULL,rescale=TRUE,...){
 
       subset<-substitute(subset)
       subset<-eval(subset, model.frame(design), parent.frame())
@@ -1262,7 +1265,8 @@ svyglm.survey.design<-function(formula,design,subset=NULL, family=stats::gaussia
       g<-match.call()
       g$formula<-eval.parent(g$formula)
       g$design<-NULL
-      g$var<-NULL
+      g$var <- NULL
+      g$rescale <- NULL
       g$family<-family
       if (is.null(g$weights))
         g$weights<-quote(.survey.prob.weights)
@@ -1272,7 +1276,9 @@ svyglm.survey.design<-function(formula,design,subset=NULL, family=stats::gaussia
       g[[1]]<-quote(glm)      
 
       ##need to rescale weights for stability in binomial
-      data$.survey.prob.weights<-(1/design$prob)/mean(1/design$prob)
+      ## (unless the user doesn't want to)
+      if (rescale)
+          data$.survey.prob.weights<-(1/design$prob)/mean(1/design$prob)
       if (!all(all.vars(formula) %in% names(data))) 
 	stop("all variables must be in design= argument")
       g<-with(list(data=data), eval(g))
@@ -1660,45 +1666,46 @@ MASSprofile_glm<-function (fitted, which = 1:p, alpha = 0.01, maxsteps = 10, del
 ###
 
 svymle<-function(loglike, gradient=NULL, design, formulas,
-                 start=NULL, control=list(maxit=1000),
-                 na.action="na.fail", method=NULL,...){
+    start=NULL, control=list(),
+    na.action="na.fail", method=NULL,lower=NULL,upper=NULL,...){
   if(is.null(method))
-    method<-if(is.null(gradient)) "Nelder-Mead" else "nlm"
-  
+    method<-if(is.null(gradient)) "Nelder-Mead" else "newuoa"
   if (!inherits(design,"survey.design")) 
-	stop("design is not a survey.design")
+    stop("design is not a survey.design")
   weights<-weights(design)
   wtotal<-sum(weights)
- 
-  if (is.null(control$fnscale))
-      control$fnscale <- -wtotal/length(weights)
-  if (inherits(design, "twophase"))
+  
+  if (!(method %in% c("bobyqa","newuoa")) && is.null(control$fnscale))
+    control$fnscale <- -wtotal/length(weights)
+  if (inherits(design, "twophase")) {
     data<-design$phase1$sample$variables
-  else 
+  } else { 
     data<-design$variables
-
-## Get the response variable
+  }
+  
+  ## Get the response variable
   nms<-names(formulas)
   if (nms[1]==""){
-	if (inherits(formulas[[1]],"formula"))
-	  y<-eval.parent(model.frame(formulas[[1]],data=data,na.action=na.pass))
-	else
-	  y<-eval(y,data,parent.frame())
-	formulas[1]<-NULL
-	if (FALSE && NCOL(y)>1) stop("Y has more than one column")
-    }   else {
-  	## one formula must have response
-	has.response<-sapply(formulas,length)==3
-	if (sum(has.response)!=1) stop("Need a response variable")
-	ff<-formulas[[which(has.response)]]
-	ff[[3]]<-1
-	y<-eval.parent(model.frame(ff,data=data,na.action=na.pass))
-	formulas[[which(has.response)]]<-delete.response(terms(formulas[[which(has.response)]]))
-        nms<-c("",nms)
+    if (inherits(formulas[[1]],"formula")) {
+      y<-eval.parent(model.frame(formulas[[1]],data=data,na.action=na.pass))
+    } else {
+      y<-eval(y,data,parent.frame())
+    }
+    formulas[1]<-NULL
+    if (FALSE && NCOL(y)>1) stop("Y has more than one column")
+  }   else {
+    ## one formula must have response
+    has.response<-sapply(formulas,length)==3
+    if (sum(has.response)!=1) stop("Need a response variable")
+    ff<-formulas[[which(has.response)]]
+    ff[[3]]<-1
+    y<-eval.parent(model.frame(ff,data=data,na.action=na.pass))
+    formulas[[which(has.response)]]<-delete.response(terms(formulas[[which(has.response)]]))
+    nms<-c("",nms)
   }
-
+  
   if(length(which(nms==""))>1) stop("Formulas must have names")
-
+  
   mf<-vector("list",length(formulas))
   vnms <- unique(do.call(c, lapply(formulas, all.vars)))
   uformula <- make.formula(vnms)
@@ -1714,305 +1721,148 @@ svymle<-function(loglike, gradient=NULL, design, formulas,
   wtotal<-sum(weights)
   
   Y<-mf[,1]
-  mm<-lapply(formulas,model.matrix, data=mf)
-
+  
+#  mm <- lapply(formulas,model.matrix, data=mf)
+  
+  mmFrame <- lapply(formulas,model.frame, data=mf)
+  mm = mapply(model.matrix, object = formulas, data=mmFrame)
+  mmOffset <- lapply(mmFrame, model.offset)
+  
+  # add a vector of zeros if there is no offset provided
+  noOffset = which(unlist(lapply(mmOffset, length))==0)
+  for(D in noOffset) {
+    mmOffset[[D]] =  rep(0, nrow(mm[[D]]))
+  }
+  
   ## parameter names
   parnms<-lapply(mm,colnames)
   for(i in 1:length(parnms))
-	parnms[[i]]<-paste(nms[i+1],parnms[[i]],sep=".")
+    parnms[[i]]<-paste(nms[i+1],parnms[[i]],sep=".")
   parnms<-unlist(parnms)
-
+  
   # maps position in theta to model matrices
   np<-c(0,cumsum(sapply(mm,NCOL)))
-
-
+  
+  
   objectivefn<-function(theta,...){
-     args<-vector("list",length(nms))
-     args[[1]]<-Y
-     for(i in 2:length(nms))
-	args[[i]]<-mm[[i-1]]%*%theta[(np[i-1]+1):np[i]]
-     names(args)<-nms
-     args<-c(args, ...)
-     sum(do.call("loglike",args)*weights)
+    args<-vector("list",length(nms))
+    args[[1]]<-Y
+    for(i in 2:length(nms))
+      args[[i]]<-mm[[i-1]]%*%theta[(np[i-1]+1):np[i]] + 
+          mmOffset[[i-1]]
+    names(args)<-nms
+    args<-c(args, ...)
+    sum(do.call("loglike",args)*weights)
   }
-
+  
   if (is.null(gradient)) {
-     grad<-NULL
+    grad<-NULL
   } else {  
-     fnargs<-names(formals(loglike))[-1]
-     grargs<-names(formals(gradient))[-1]
-     if(!identical(fnargs,grargs))
-       stop("loglike and gradient have different arguments.")
-     reorder<-na.omit(match(grargs,nms[-1]))
-     grad<-function(theta,...){
-       args<-vector("list",length(nms))
-       args[[1]]<-Y
-       for(i in 2:length(nms))
-	  args[[i]]<-drop(mm[[i-1]]%*%theta[(np[i-1]+1):np[i]])
-       names(args)<-nms
-       args<-c(args,...)
-       rval<-NULL
-       tmp<-do.call("gradient",args)
-       for(i in reorder){
-	   rval<-c(rval, colSums(as.matrix(tmp[,i]*weights*mm[[i]])))
-	}
-       drop(rval)
-     }
+    fnargs<-names(formals(loglike))[-1]
+    grargs<-names(formals(gradient))[-1]
+    if(!identical(fnargs,grargs))
+      stop("loglike and gradient have different arguments.")
+    reorder<-na.omit(match(grargs,nms[-1]))
+    grad<-function(theta,...){
+      args<-vector("list",length(nms))
+      args[[1]]<-Y
+      for(i in 2:length(nms))
+        args[[i]]<-drop(
+            mm[[i-1]]%*%theta[(np[i-1]+1):np[i]] +
+                mmOffset[[i-1]])
+      names(args)<-nms
+      args<-c(args,...)
+      rval<-NULL
+      tmp<-do.call("gradient",args)
+      for(i in reorder){
+        rval<-c(rval, colSums(as.matrix(tmp[,i]*weights*mm[[i]])))
+      }
+      drop(rval)
+    }
   }
-
+  
   theta0<-numeric(np[length(np)])
   if (is.list(start))
-      st<-do.call("c",start)
+    st<-do.call("c",start)
   else
-      st<-start
-
+    st<-start
+  
   if (length(st)==length(theta0)) {
-	theta0<-st
+    theta0<-st
   } else {
-	stop("starting values wrong length")
+    stop("starting values wrong length")
   }
-
+  
   if (method=="nlm"){
-      ff<-function(theta){
-          rval<- -objectivefn(theta)
-          if (is.na(rval)) rval<- -Inf
-          attr(rval,"gradient")<- -grad(theta)
-          rval
-      }
-      rval<-nlm(ff, theta0,hessian=TRUE)
-      if (rval$code>3) warning("nlm did not converge")
-      rval$par<-rval$estimate
-  } else {
-      rval<-optim(theta0, objectivefn, grad,control=control,
-              hessian=TRUE,method=method,...)
-      if (rval$conv!=0) warning("optim did not converge")
+    ff<-function(theta){
+      rval<- -objectivefn(theta)
+      if (is.na(rval)) rval<- -Inf
+      attr(rval,"gradient")<- -grad(theta)
+      rval
+    }
+    rval<-nlm(ff, theta0,hessian=TRUE)
+    if (rval$code>3) warning("nlm did not converge")
+    rval$par<-rval$estimate
+  } else if (method == "newuoa"){
+      rval<-minqa::uobyqa(theta0, function(par,...) -objectivefn(par,...), control=control, ...)
+      if(rval$ier>0) warning(rval$msg)
+      rval$hessian <- numDeriv::hessian(objectivefn, rval$par)
+  } else if (method == "bobyqa"){
+      rval<-minqa::bobyqa(theta0, function(par,...) -objectivefn(par,...), control=control,lower=lower,upper=upper, ...)
+      if(rval$ier>0) warning(rval$msg)
+      rval$hessian <- numDeriv::hessian(objectivefn,rval$par)
+  }else {
+    rval<-optim(theta0, objectivefn, grad,control=control,
+        hessian=TRUE,method=method,...)
+    if (rval$conv!=0) warning("optim did not converge")
   }
-
+  
+  
+  
   
-
-
   names(rval$par)<-parnms
   dimnames(rval$hessian)<-list(parnms,parnms)
-
+  
   if (is.null(gradient)) {
-	rval$invinf<-solve(-rval$hessian)
-	rval$scores<-NULL
-	rval$sandwich<-NULL
-    }  else {
-       theta<-rval$par
-       args<-vector("list",length(nms))
-       args[[1]]<-Y
-       for(i in 2:length(nms))
-	  args[[i]]<-drop(mm[[i-1]]%*%theta[(np[i-1]+1):np[i]])
-       names(args)<-nms
-       args<-c(args,...)
-       deta<-do.call("gradient",args)
-       rval$scores<-NULL
-       for(i in reorder)
-       	 rval$scores<-cbind(rval$scores,deta[,i]*weights*mm[[i]])
-
-       rval$invinf<-solve(-rval$hessian)
-       dimnames(rval$invinf)<-list(parnms,parnms)
-
-       db<-rval$scores%*%rval$invinf
-       if (inherits(design,"survey.design2"))
-         rval$sandwich<-svyrecvar(db,design$cluster,design$strata, design$fpc, 
-                               postStrata=design$postStrata)
-       else if (inherits(design, "twophase"))
-         rval$sandwich<-twophasevar(db,design)
-       else
-         rval$sandwich<-svyCprod(db,design$strata,design$cluster[[1]],
-                                 design$fpc, design$nPSU,
-                                 design$certainty, design$postStrata)
-       dimnames(rval$sandwich)<-list(parnms,parnms)
-     }
+    rval$invinf<-solve(-rval$hessian)
+    rval$scores<-NULL
+    rval$sandwich<-NULL
+  }  else {
+    theta<-rval$par
+    args<-vector("list",length(nms))
+    args[[1]]<-Y
+    for(i in 2:length(nms))
+      args[[i]]<-drop(mm[[i-1]]%*%theta[(np[i-1]+1):np[i]]+ mmOffset[[i-1]])
+    names(args)<-nms
+    args<-c(args,...)
+    deta<-do.call("gradient",args)
+    rval$scores<-NULL
+    for(i in reorder)
+      rval$scores<-cbind(rval$scores,deta[,i]*weights*mm[[i]])
+    
+    rval$invinf<-solve(-rval$hessian)
+    dimnames(rval$invinf)<-list(parnms,parnms)
+    
+    db<-rval$scores%*%rval$invinf
+    if (inherits(design,"survey.design2"))
+      rval$sandwich<-svyrecvar(db,design$cluster,design$strata, design$fpc, 
+          postStrata=design$postStrata)
+    else if (inherits(design, "twophase"))
+      rval$sandwich<-twophasevar(db,design)
+    else
+      rval$sandwich<-svyCprod(db,design$strata,design$cluster[[1]],
+          design$fpc, design$nPSU,
+          design$certainty, design$postStrata)
+    dimnames(rval$sandwich)<-list(parnms,parnms)
+  }
   rval$call<-match.call()
   rval$design<-design
   class(rval)<-"svymle"
   rval
-
+  
 }
 
 
-svymleOLD<-function(loglike, gradient=NULL, design, formulas,
-                 start=NULL, control=list(maxit=1000),
-                 na.action="na.fail", method=NULL,...){
-  if(is.null(method))
-    method<-if(is.null(gradient)) "Nelder-Mead" else "nlm"
-  
-  if (!inherits(design,"survey.design")) 
-	stop("design is not a survey.design")
-
-  weights<-1/design$prob
-  wtotal<-sum(weights)
-  if (is.null(control$fnscale))
-      control$fnscale<- -wtotal
-  if (inherits(design, "twophase"))
-    data<-design$phase1$sample$variables
-  else 
-    data<-design$variables
-
-## Get the response variable
-  nms<-names(formulas)
-  if (nms[1]==""){
-	if (inherits(formulas[[1]],"formula"))
-	  y<-eval.parent(model.frame(formulas[[1]],data=data,na.action=na.pass))
-	else
-	  y<-eval(y,data,parent.frame())
-	formulas[1]<-NULL
-	if (FALSE && NCOL(y)>1) stop("Y has more than one column")
-    }   else {
-  	## one formula must have response
-	has.response<-sapply(formulas,length)==3
-	if (sum(has.response)!=1) stop("Need a response variable")
-	ff<-formulas[[which(has.response)]]
-	ff[[3]]<-1
-	y<-eval.parent(model.frame(ff,data=data,na.action=na.pass))
-	formulas[[which(has.response)]]<-delete.response(terms(formulas[[which(has.response)]]))
-        nms<-c("",nms)
-  }
-
-  if(length(which(nms==""))>1) stop("Formulas must have names")
-  
-  
-  mf<-vector("list",length(formulas))
-  for(i in 1:length(formulas)){
-	mf[[i]]<-eval.parent(model.frame(formulas[[i]], data=data, na.action=na.pass))
-	}
-  notnulls<-sapply(mf,function(mfi) NCOL(mfi)!=0)
-  mf<-mf[notnulls]
-  if (any(notnulls))
-    mf<-as.data.frame(do.call("cbind",c(y,mf)))
-  else
-    mf<-y
-  names(mf)[1]<-"(Response)"
-  mf<-mf[,!duplicated(colnames(mf)),drop=FALSE]
-
-  mf<-get(na.action)(mf)  
-  nas<-attr(mf,"na.action")
-  if (length(nas))
-	design<-design[-nas,]
-
-  Y<-mf[,1]
-  mm<-lapply(formulas,model.matrix, data=mf)
-
-  ## parameter names
-  parnms<-lapply(mm,colnames)
-  for(i in 1:length(parnms))
-	parnms[[i]]<-paste(nms[i+1],parnms[[i]],sep=".")
-  parnms<-unlist(parnms)
-
-  # maps position in theta to model matrices
-  np<-c(0,cumsum(sapply(mm,NCOL)))
-
-
-  objectivefn<-function(theta,...){
-     args<-vector("list",length(nms))
-     args[[1]]<-Y
-     for(i in 2:length(nms))
-	args[[i]]<-mm[[i-1]]%*%theta[(np[i-1]+1):np[i]]
-     names(args)<-nms
-     args<-c(args, ...)
-     sum(do.call("loglike",args)*weights)
-  }
-
-  if (is.null(gradient)) {
-     grad<-NULL
-  } else {  
-     fnargs<-names(formals(loglike))[-1]
-     grargs<-names(formals(gradient))[-1]
-     if(!identical(fnargs,grargs))
-       stop("loglike and gradient have different arguments.")
-     reorder<-na.omit(match(grargs,nms[-1]))
-     grad<-function(theta,...){
-       args<-vector("list",length(nms))
-       args[[1]]<-Y
-       for(i in 2:length(nms))
-	  args[[i]]<-drop(mm[[i-1]]%*%theta[(np[i-1]+1):np[i]])
-       names(args)<-nms
-       args<-c(args,...)
-       rval<-NULL
-       tmp<-do.call("gradient",args)
-       for(i in reorder){
-	   rval<-c(rval, colSums(as.matrix(tmp[,i]*weights*mm[[i]])))
-	}
-       drop(rval)
-     }
-  }
-
-  theta0<-numeric(np[length(np)])
-  if (is.list(start))
-      st<-do.call("c",start)
-  else
-      st<-start
-
-  if (length(st)==length(theta0)) {
-	theta0<-st
-  } else {
-	stop("starting values wrong length")
-  }
-
-  if (method=="nlm"){
-      ff<-function(theta){
-          rval<- -objectivefn(theta)
-          if (is.na(rval)) rval<- -Inf
-          attr(rval,"grad")<- -grad(theta)
-          rval
-      }
-      rval<-nlm(ff, theta0,hessian=TRUE)
-      if (rval$code>3) warning("nlm did not converge")
-      rval$par<-rval$estimate
-  } else {
-      rval<-optim(theta0, objectivefn, grad,control=control,
-              hessian=TRUE,method=method,...)
-      if (rval$conv!=0) warning("optim did not converge")
-  }
-
-  
-
-
-  names(rval$par)<-parnms
-  dimnames(rval$hessian)<-list(parnms,parnms)
-
-  if (is.null(gradient)) {
-	rval$invinf<-solve(-rval$hessian)
-	rval$scores<-NULL
-	rval$sandwich<-NULL
-    }  else {
-       theta<-rval$par
-       args<-vector("list",length(nms))
-       args[[1]]<-Y
-       for(i in 2:length(nms))
-	  args[[i]]<-drop(mm[[i-1]]%*%theta[(np[i-1]+1):np[i]])
-       names(args)<-nms
-       args<-c(args,...)
-       deta<-do.call("gradient",args)
-       rval$scores<-NULL
-       for(i in reorder)
-       	 rval$scores<-cbind(rval$scores,deta[,i]*weights*mm[[i]])
-
-       rval$invinf<-solve(-rval$hessian)
-       dimnames(rval$invinf)<-list(parnms,parnms)
-
-       db<-rval$scores%*%rval$invinf
-       if (inherits(design,"survey.design2"))
-         rval$sandwich<-svyrecvar(db,design$cluster,design$strata, design$fpc, 
-                               postStrata=design$postStrata)
-       else if (inherits(design, "twophase"))
-         rval$sandwich<-twophasevar(db,design)
-       else
-         rval$sandwich<-svyCprod(db,design$strata,design$cluster[[1]],
-                                 design$fpc, design$nPSU,
-                                 design$certainty, design$postStrata)
-       dimnames(rval$sandwich)<-list(parnms,parnms)
-     }
-  rval$call<-match.call()
-  rval$design<-design
-  class(rval)<-"svymle"
-  rval
-
-}
-
 coef.svymle<-function(object,...) object$par
 
 vcov.svymle<-function(object,stderr=c("robust","model"),...) {
